#!BPY
# GenBlend v0.2 - by D. R. Gunter(a.k.a. Graywolf)
# Matrix math and and blender bone conversion borrowed from:
# blender2cal3D.py version 0.5
# which is:
# Copyright (C) 2003 Jean-Baptiste LAMY -- jiba@tuxfamily.org
#
#	Thank you, Jean-Baptiste. The bones were giving me hell...
#
# And, considering that, this must be stated:
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
Name: 'Genesis3d Actor Export'
Blender: 246
Group: 'Export'
Tooltip: 'Export to Genesis3d .nfo/.key files.'
"""

# This script is a Blender 2.46 => Genesis3D nfo/key file exporter.
# (See http://blender.org and http://www.realityfactory.info)
#
# Notes(NFO export):
#		1. The script only exports "selected" objects to an NFO.
#			This provides a minor effeciency boost. It also allows you
#			to use an existing model and skeleton for crafting/aligning/
#			exporting attachment type actors.
#
#		2. Only one armature should be selected, as only the first arm-
#			ature encountered will be processed... Floating bones within
#			the armature are perfectly fine, however. They will simply
#			be parented to the root.
#
#		3. Don't use vertex group weighting. Each vertex should belong to only
#			one group, otherwise strange things might(will..) occur.
#
#		4. For materials, if an image is assigned, the name of the material
#			will be the name of the image datablock(as assigned in the UV editor),
#			otherwise the material datablock name will be used.
#
#		5. For the indefinite future, UVs are per face only. No stickies.
#
#
# Grab the latest version here :
# htpu://WWWhere?.2Bdetermined.2do

# And so it begins...

import sys, os, os.path, math, string, inspect, shutil
import Blender
from Blender import Draw, Window

# Math stuff

def quaternion2matrix(q):
  xx = q[0] * q[0]
  yy = q[1] * q[1]
  zz = q[2] * q[2]
  xy = q[0] * q[1]
  xz = q[0] * q[2]
  yz = q[1] * q[2]
  wx = q[3] * q[0]
  wy = q[3] * q[1]
  wz = q[3] * q[2]
  return [[1.0 - 2.0 * (yy + zz),       2.0 * (xy + wz),       2.0 * (xz - wy), 0.0],
          [      2.0 * (xy - wz), 1.0 - 2.0 * (xx + zz),       2.0 * (yz + wx), 0.0],
          [      2.0 * (xz + wy),       2.0 * (yz - wx), 1.0 - 2.0 * (xx + yy), 0.0],
          [0.0                  , 0.0                  , 0.0                  , 1.0]]

def matrix2quaternion(m):
  s = math.sqrt(abs(m[0][0] + m[1][1] + m[2][2] + m[3][3]))
  if s == 0.0:
    x = abs(m[2][1] - m[1][2])
    y = abs(m[0][2] - m[2][0])
    z = abs(m[1][0] - m[0][1])
    if   (x >= y) and (x >= z): return 1.0, 0.0, 0.0, 0.0
    elif (y >= x) and (y >= z): return 0.0, 1.0, 0.0, 0.0
    else:                       return 0.0, 0.0, 1.0, 0.0
  return quaternion_normalize([
    -(m[2][1] - m[1][2]) / (2.0 * s),
    -(m[0][2] - m[2][0]) / (2.0 * s),
    -(m[1][0] - m[0][1]) / (2.0 * s),
    0.5 * s,
    ])

def quaternion_normalize(q):
  l = math.sqrt(q[0] * q[0] + q[1] * q[1] + q[2] * q[2] + q[3] * q[3])
  return q[0] / l, q[1] / l, q[2] / l, q[3] / l

def quaternion_multiply(q1, q2):
  r = [
    q2[3] * q1[0] + q2[0] * q1[3] + q2[1] * q1[2] - q2[2] * q1[1],
    q2[3] * q1[1] + q2[1] * q1[3] + q2[2] * q1[0] - q2[0] * q1[2],
    q2[3] * q1[2] + q2[2] * q1[3] + q2[0] * q1[1] - q2[1] * q1[0],
    q2[3] * q1[3] - q2[0] * q1[0] - q2[1] * q1[1] - q2[2] * q1[2],
    ]
  d = math.sqrt(r[0] * r[0] + r[1] * r[1] + r[2] * r[2] + r[3] * r[3])
  r[0] /= d
  r[1] /= d
  r[2] /= d
  r[3] /= d
  return r

def matrix_translate(m, v):
  m[3][0] += v[0]
  m[3][1] += v[1]
  m[3][2] += v[2]
  return m

def matrix_multiply(b, a):
  return [ [
    a[0][0] * b[0][0] + a[0][1] * b[1][0] + a[0][2] * b[2][0],
    a[0][0] * b[0][1] + a[0][1] * b[1][1] + a[0][2] * b[2][1],
    a[0][0] * b[0][2] + a[0][1] * b[1][2] + a[0][2] * b[2][2],
    0.0,
    ], [
    a[1][0] * b[0][0] + a[1][1] * b[1][0] + a[1][2] * b[2][0],
    a[1][0] * b[0][1] + a[1][1] * b[1][1] + a[1][2] * b[2][1],
    a[1][0] * b[0][2] + a[1][1] * b[1][2] + a[1][2] * b[2][2],
    0.0,
    ], [
    a[2][0] * b[0][0] + a[2][1] * b[1][0] + a[2][2] * b[2][0],
    a[2][0] * b[0][1] + a[2][1] * b[1][1] + a[2][2] * b[2][1],
    a[2][0] * b[0][2] + a[2][1] * b[1][2] + a[2][2] * b[2][2],
     0.0,
    ], [
    a[3][0] * b[0][0] + a[3][1] * b[1][0] + a[3][2] * b[2][0] + b[3][0],
    a[3][0] * b[0][1] + a[3][1] * b[1][1] + a[3][2] * b[2][1] + b[3][1],
    a[3][0] * b[0][2] + a[3][1] * b[1][2] + a[3][2] * b[2][2] + b[3][2],
    1.0,
    ] ]

def matrix_invert(m):
  det = (m[0][0] * (m[1][1] * m[2][2] - m[2][1] * m[1][2])
       - m[1][0] * (m[0][1] * m[2][2] - m[2][1] * m[0][2])
       + m[2][0] * (m[0][1] * m[1][2] - m[1][1] * m[0][2]))
  if det == 0.0: return None
  det = 1.0 / det
  r = [ [
      det * (m[1][1] * m[2][2] - m[2][1] * m[1][2]),
    - det * (m[0][1] * m[2][2] - m[2][1] * m[0][2]),
      det * (m[0][1] * m[1][2] - m[1][1] * m[0][2]),
      0.0,
    ], [
    - det * (m[1][0] * m[2][2] - m[2][0] * m[1][2]),
      det * (m[0][0] * m[2][2] - m[2][0] * m[0][2]),
    - det * (m[0][0] * m[1][2] - m[1][0] * m[0][2]),
      0.0
    ], [
      det * (m[1][0] * m[2][1] - m[2][0] * m[1][1]),
    - det * (m[0][0] * m[2][1] - m[2][0] * m[0][1]),
      det * (m[0][0] * m[1][1] - m[1][0] * m[0][1]),
      0.0,
    ] ]
  r.append([
    -(m[3][0] * r[0][0] + m[3][1] * r[1][0] + m[3][2] * r[2][0]),
    -(m[3][0] * r[0][1] + m[3][1] * r[1][1] + m[3][2] * r[2][1]),
    -(m[3][0] * r[0][2] + m[3][1] * r[1][2] + m[3][2] * r[2][2]),
    1.0,
    ])
  return r

def matrix_rotate_x(angle):
  cos = math.cos(angle)
  sin = math.sin(angle)
  return [
    [1.0,  0.0, 0.0, 0.0],
    [0.0,  cos, sin, 0.0],
    [0.0, -sin, cos, 0.0],
    [0.0,  0.0, 0.0, 1.0],
    ]

def matrix_rotate_y(angle):
  cos = math.cos(angle)
  sin = math.sin(angle)
  return [
    [cos, 0.0, -sin, 0.0],
    [0.0, 1.0,  0.0, 0.0],
    [sin, 0.0,  cos, 0.0],
    [0.0, 0.0,  0.0, 1.0],
    ]

def matrix_rotate_z(angle):
  cos = math.cos(angle)
  sin = math.sin(angle)
  return [
    [ cos, sin, 0.0, 0.0],
    [-sin, cos, 0.0, 0.0],
    [ 0.0, 0.0, 1.0, 0.0],
    [ 0.0, 0.0, 0.0, 1.0],
    ]

def matrix_rotate(axis, angle):
  vx  = axis[0]
  vy  = axis[1]
  vz  = axis[2]
  vx2 = vx * vx
  vy2 = vy * vy
  vz2 = vz * vz
  cos = math.cos(angle)
  sin = math.sin(angle)
  co1 = 1.0 - cos
  return [
    [vx2 * co1 + cos,          vx * vy * co1 + vz * sin, vz * vx * co1 - vy * sin, 0.0],
    [vx * vy * co1 - vz * sin, vy2 * co1 + cos,          vy * vz * co1 + vx * sin, 0.0],
    [vz * vx * co1 + vy * sin, vy * vz * co1 - vx * sin, vz2 * co1 + cos,          0.0],
    [0.0, 0.0, 0.0, 1.0],
    ]

def matrix_scale(fx, fy, fz):
  return [
    [ fx, 0.0, 0.0, 0.0],
    [0.0,  fy, 0.0, 0.0],
    [0.0, 0.0,  fz, 0.0],
    [0.0, 0.0, 0.0, 1.0],
    ]
  
def point_by_matrix(p, m):
  return [p[0] * m[0][0] + p[1] * m[1][0] + p[2] * m[2][0] + m[3][0],
          p[0] * m[0][1] + p[1] * m[1][1] + p[2] * m[2][1] + m[3][1],
          p[0] * m[0][2] + p[1] * m[1][2] + p[2] * m[2][2] + m[3][2]]

def point_distance(p1, p2):
  return math.sqrt((p2[0] - p1[0]) ** 2 + (p2[1] - p1[1]) ** 2 + (p2[2] - p1[2]) ** 2)

def vector_sub(v1, v2):
	return [v1[0]-v2[0], v1[1]-v2[1], v1[2]-v2[2]]
  
def vector_by_matrix(p, m):
  return [p[0] * m[0][0] + p[1] * m[1][0] + p[2] * m[2][0],
          p[0] * m[0][1] + p[1] * m[1][1] + p[2] * m[2][1],
          p[0] * m[0][2] + p[1] * m[1][2] + p[2] * m[2][2]]

def vector_length(v):
  return math.sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2])

def vector_normalize(v):
  l = math.sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2])
  return v[0] / l, v[1] / l, v[2] / l

def vector_dotproduct(v1, v2):
  return v1[0] * v2[0] + v1[1] * v2[1] + v1[2] * v2[2]

def vector_crossproduct(v1, v2):
  return [
    v1[1] * v2[2] - v1[2] * v2[1],
    v1[2] * v2[0] - v1[0] * v2[2],
    v1[0] * v2[1] - v1[1] * v2[0],
    ]

def vector_angle(v1, v2):
  s = vector_length(v1) * vector_length(v2)
  f = vector_dotproduct(v1, v2) / s
  if f >=  1.0: return 0.0
  if f <= -1.0: return math.pi / 2.0
  return math.atan(-f / math.sqrt(1.0 - f * f)) + math.pi / 2.0

def blender_bone2matrix(head, tail, roll):
  # Convert bone rest state (defined by bone.head, bone.tail and bone.roll)
  # to a matrix (the more standard notation).
  # Taken from blenkernel/intern/armature.c in Blender source.
  # See also DNA_armature_types.h:47.
  
  target = [0.0, 1.0, 0.0]
  delta  = [tail[0] - head[0], tail[1] - head[1], tail[2] - head[2]]
  nor    = vector_normalize(delta)
  axis   = vector_crossproduct(target, nor)
  
  if vector_dotproduct(axis, axis) > 0.0000000000001:
    axis    = vector_normalize(axis)
    theta   = math.acos(vector_dotproduct(target, nor))
    bMatrix = matrix_rotate(axis, theta)
    
  else:
    if vector_crossproduct(target, nor) > 0.0: updown =  1.0
    else:                                      updown = -1.0
    
    # Quoted from Blender source : "I think this should work ..."
    bMatrix = [
      [updown, 0.0, 0.0, 0.0],
      [0.0, updown, 0.0, 0.0],
      [0.0, 0.0, 1.0, 0.0],
      [0.0, 0.0, 0.0, 1.0],
      ]
  
  rMatrix = matrix_rotate(nor, roll)
  return matrix_multiply(rMatrix, bMatrix)
  
def blender_bone2matrix(head, tail, roll):
	# Convert bone rest state (defined by bone.head, bone.tail and bone.roll)
	# to a matrix (the more standard notation).
	# Taken from blenkernel/intern/armature.c in Blender source.
	# See also DNA_armature_types.h:47.
	
	target = [0.0, 1.0, 0.0]
	delta  = [tail[0] - head[0], tail[1] - head[1], tail[2] - head[2]]
	nor    = vector_normalize(delta)
	axis   = vector_crossproduct(target, nor)
	
	if vector_dotproduct(axis, axis) > 0.0000000000001:
		axis    = vector_normalize(axis)
		theta   = math.acos(vector_dotproduct(target, nor))
		bMatrix = matrix_rotate(axis, math.degrees(theta))
		
	else:
		if vector_dotproduct(target, nor) > 0.0: updown =  1.0
		else:                                    updown = -1.0
		
		# Quoted from Blender source : "I think this should work ..."
		bMatrix = [
			[updown, 0.0,    0.0, 0.0],
			[0.0,    updown, 0.0, 0.0],
			[0.0,    0.0,    1.0, 0.0],
			[0.0,    0.0,    0.0, 1.0],
			]
	
	rMatrix = matrix_rotate(nor, roll)
	return matrix_multiply(rMatrix, bMatrix)

class NTM:
	def __init__(self, parent, name, loc, rot):
		self.parent = parent
		self.name   = name
		self.loc = loc
		self.rot = rot
		self.children = []
		
		self.matrix = matrix_translate(quaternion2matrix(rot), loc)
		
		if parent:
			self.matrix = matrix_multiply(parent.matrix, self.matrix)
			parent.children.append(self)
		
		# lloc and lrot are the bone => model space transformation (translation and rotation).
		# They are probably specific to Cal3D.
		m = matrix_invert(self.matrix)
		self.lloc = m[3][0], m[3][1], m[3][2]
		self.lrot = matrix2quaternion(m)
		
		self.scale = m[0][0], m[1][1], m[2][2]

class NFOData(list):
	ARMATURE = None
	MAT_INDEX = []
	MAT_DATA = {}
	BONE_INDEX = []
	NTM_INDEX = []
	IMAGE_FILEPATHS = []
	NTM_DATA = {}
	NTM_LIST = {}
		
def FormatBones(NFO):
	matrix = NFO.ARMATURE.matrix
	#if BASE_MATRIX:
	#	matrix = matrix_multiply(BASE_MATRIX, matrix)
	
	def treat_bone(b, BC, parent = None):
		head = b.head["BONESPACE"]
		tail = b.tail["BONESPACE"]
		
		# Turns the Blender's head-tail-roll notation into a quaternion
		quat = matrix2quaternion(blender_bone2matrix(head, tail, b.roll["BONESPACE"]))
		
		if b.parent:
			# Compute the translation from the parent bone's head to the child
			# bone's head, in the parent bone coordinate system.
			# The translation is parent_tail - parent_head + child_head,
			# but parent_tail and parent_head must be converted from the parent's parent
			# system coordinate into the parent system coordinate.
			
			parent_invert_transform = matrix_invert(quaternion2matrix(parent.rot))
			parent_head = vector_by_matrix(parent.head, parent_invert_transform)
			parent_tail = vector_by_matrix(parent.tail, parent_invert_transform)
			
			parentheadtotail = vector_sub(parent_tail, parent_head)
			loc = parentheadtotail
			rot = quat
			
			NFO.NTM_LIST[b.name] = NTM(parent, b.name, loc, rot)
			NFO.BONE_INDEX.append(b)
			
		else:
			# Apply the armature's matrix to the root bones
			head = point_by_matrix(head, matrix)
			tail = point_by_matrix(tail, matrix)
			quat = matrix2quaternion(matrix_multiply(matrix, quaternion2matrix(quat))) # Probably not optimal
			
			# loc = vector_add(head, b.getLoc())
			# rot = quaternion_multiply(blender2cal3dquat(b.getQuat()), quat)
			loc = head
			rot = quat 
			
			# Here, the translation is simply the head vector
			NFO.NTM_LIST[b.name] = NTM(None, b.name, loc, rot)
			NFO.BONE_INDEX.append(b)
		
		NFO.NTM_LIST[b.name].head = head
		NFO.NTM_LIST[b.name].tail = tail
		
		if b.hasChildren():
			for child in b.children:
				treat_bone(child, BC, NFO.NTM_LIST[b.name])
				
	for b in NFO.ARMATURE.data.bones.values():
		# child bones are handled in treat_bone
		if not b.parent:
			treat_bone(b, 0)

def FormatNTMS(NFO):
	# Then take care of the rest
	for NTMName in NFO.NTM_INDEX:
		NTMData = NFO.NTM_DATA[NTMName]
		if type(NTMData) is Blender.Types.ObjectType:
			matrix = NTMData.matrix
			quat = matrix2quaternion(matrix)
			size = NTMData.size
			loc = NTMData.loc
			NFO.NTM_LIST[NTMName] = NTM(None, NTMName, loc, quat)
	
def ProcessMats(NFO):
	NFO.append("Material List")
	NFO.append("Number of Materials = %i" % len(NFO.MAT_INDEX))
	for Mat in NFO.MAT_INDEX:
		if type(NFO.MAT_DATA[Mat]) is Blender.Types.ImageType:
			Img = NFO.MAT_DATA[Mat]
			Filepath = Img.filename
			NFO.IMAGE_FILEPATHS.append(Filepath)
			Filename = os.path.split(Filepath)[1]
			NFO.append("(MAP) %s: %s" % (Img.name, Filename))
		else:
			RGB = NFO.MAT_DATA[Mat].getRGBCol()
			NFO.append("(RGB) %s: %f %f %f" % (Mat, RGB[0], RGB[1], RGB[2]))

def ProcessSkeleton(NFO):
	ROOTBONESET = False
	FormatBones(NFO)
	FormatNTMS(NFO)
	NFO.append("Bone List")
	NFO.append("Number of Bones = %i" % len(NFO.BONE_INDEX))
	for Bone in NFO.BONE_INDEX:
		Boneparentindex = -1
		NFO.append("Bone: %s" % Bone.name)
		if Bone.parent:
			Boneparentindex = NFO.BONE_INDEX.index(Bone.parent)
		elif ROOTBONESET:
			Boneparentindex = 0
		else:
			ROOTBONESET = True
		NFO.append("Parent: %i" % Boneparentindex)
	# Transformation information
	NFO.append("Node Transform Matrix List")
	NFO.append("Number of Nodes = %i" % len(NFO.NTM_INDEX))
	i = 0
	for k, v in NFO.NTM_LIST.iteritems():
		NFO.append("Node(%i): %s" % (i, v.name))
		for line in v.matrix:
			NFO.append("%f, %f, %f," % (line[0], line[1], line[2]))
		NFO.append("Q(w,x,y,z): %f %f %f %f" % (v.rot[3], v.rot[0], v.rot[1], v.rot[2]))
		NFO.append("S(x,y,z): %f %f %f" % (v.scale[0], v.scale[1], v.scale[2]))
		NFO.append("T(x,y,z): %f %f %f" % (v.loc[0], v.loc[1], v.loc[2]))
		i = i + 1

def ProcessVerts(NFO, Mesh):
	NFO.append("Node: %s" % Mesh.name)
	# List'um da vertices
	NFO.append("Vertex List")
	NFO.append("Number of Vertices = %i" % len(Mesh.verts))
	for Vert in Mesh.verts:
		Bonegroup = Mesh.getVertexInfluences(Vert.index)
		if Bonegroup:
			Bonename = Bonegroup[0][0]
			Bone = NFO.NTM_LIST[Bonename]
			off = [0,0,0]
			for i in range(2):
				off[i] = Bone.loc[i] - Vert.co[i]
		else:
			Bonename = Mesh.name
			off = Vert.co
		pos = Vert.co
		NFO.append("%f %f %f %f %f %f %s" % (pos[0], pos[1], pos[2], off[0], off[1], off[2], Bonename))

def ProcessFaces(NFO, Mesh):
	# Plug in some UVs and faces
	uvc = 0
	fc = 0
	NFO_FACES = []
	NFO_UVS = []
	for Face in Mesh.faces:
		NFO_FACES.append("Face %i:" % fc)
		fc += 1
		if len(Face.v) != 3:
			raise ValueError("Mesh %s is not triangulated!" % Mesh.name)
		if Face.image:
			if Face.image.name in NFO.MAT_INDEX:
				NFO_FACES.append(NFO.MAT_INDEX.index(Face.image.name))
		elif Mesh.materials[Face.mat]:
			Mat = Mesh.materials[Face.mat]
			if Mat.name in NFO.MAT_INDEX:
				NFO_FACES.append(NFO.MAT_INDEX.index(Mat.name))
		if Face.smooth:
			NFO_FACES.append("1")
		else:
			NFO_FACES.append("0")
		verts = Face.v
		uvs = Face.uv
		for uv in uvs:
			NFO_UVS.append("%f %f" % uv)
		for v in verts:
			NFO_FACES.append("%i %i" % (v.index, uvc))
			uvc = uvc + 1
	NFO.append("Texture Vertex List")
	NFO.append("Number of Texture Vertices = %i" % len(NFO_UVS))
	NFO.extend(NFO_UVS)
	NFO.append("Face List")
	NFO.append("Number of Faces = %i" % len(Mesh.faces))
	NFO.extend(NFO_FACES)

def ProcessNFO(OBJS):
	NFO = NFOData()
	NFO.append("NFO 2.0")
	MESHES = []	
	# Do some front work...
	for Obj in OBJS:
		Data = Obj.getData()
		if type(Data) is Blender.Types.ArmatureType:
			NFO.ARMATURE = Obj
			n = NFO.ARMATURE.data.bones.keys()
			for name in n:
				NFO.NTM_INDEX.append(name)
				NFO.NTM_DATA[name] = NFO.ARMATURE.data.bones[name]
		elif type(Data) is Blender.Types.NMeshType:
			MESHES.append(Data)
			NFO.NTM_INDEX.append(Data.name)
			NFO.NTM_DATA[Data.name] = Obj
		else:
			print "Skipping unexportable object %s.\n" % obj.getName()
	if not NFO.ARMATURE:
		raise StandardError("No armature selected!")
	for Mesh in MESHES:
		for Face in Mesh.faces:
			if Face.image:
				Img = Face.image
				if Img.name not in NFO.MAT_INDEX:
					NFO.MAT_INDEX.append(Img.name)
					NFO.MAT_DATA[Img.name] = Img
			elif Mesh.materials[Face.mat]:
				Mat = Mesh.materials[Face.mat]
				if Mat.name not in NFO.MAT_INDEX:
					NFO.MAT_INDEX.append(Mat.name)
					NFO.MAT_DATA[Mat.name] = Mat
			else:
				raise ValueError("Error in %s: No material for face %i." % (Mesh.name, Mesh.faces.index(Face)))
	ProcessMats(NFO)
	ProcessSkeleton(NFO)
	NFO.append("Mesh List")
	NFO.append("Number of meshes = %i" % len(MESHES))
	for Mesh in MESHES:
		ProcessVerts(NFO, Mesh)
		ProcessFaces(NFO, Mesh)
	return NFO

NFO_FILENAME = Draw.Create("./Actor.nfo")
COPY_IMAGES = True

def SetExportFilename(filename):
	global NFO_FILENAME
	print type(filename)
	print filename
	NFO_FILENAME.val = filename

def SaveNFOFile(NFO):
	global NFO_FILENAME
	try:
		nfo_name = str(NFO_FILENAME.val)
		print nfo_name
		nfo_ext = os.path.splitext(nfo_name)
		if nfo_ext[1] != ".nfo":
			nfo_name = nfo_ext[0] + ".nfo"
		nfo_file = open(nfo_name, 'w')
		EXPORT_PATH = os.path.split(nfo_name)[0]
		for line in NFO:
			print >> nfo_file, line
		nfo_file.close()
		if COPY_IMAGES:
			for nfo_image in NFO.IMAGE_FILEPATHS:
				image_filename = os.path.split(nfo_image)[1]
				shutil.copyfile(nfo_image, os.path.join(EXPORT_PATH, image_filename))
	except StandardError, e:
		print str(e)
		raise
		
def GUIDraw():
	try:
		global NFO_FILENAME
		global COPY_IMAGES
		Draw.Button("Export NFO", 1, 5, 55, 100, 25, "Export NFO file.")
		NFO_FILENAME = Draw.String("Export Directory: ", 2, 5, 25, 350, 25, NFO_FILENAME.val, 256, "File to export.")
		Draw.Button("...", 3, 355, 25, 20, 25, "Browse to select export location.")
		Draw.Toggle("Copy Material Images", 4, 110, 55, 128, 25, COPY_IMAGES, "Copy texture image files when exporting.")
		Draw.Button("Exit", 5, 250, 55, 64, 25, "Close Exporter")
	except StandardError, e:
		print str(e)
		Draw.Exit()
		raise
		
	
def GUIInputEvent(event, value):
	try:
		if not value:
			if event == Draw.ESCKEY:
				Draw.Exit()
				return
	except StandardError, e:
		print str(e)
		Draw.Exit()
		raise
  	  
def GUIButtonEvent(event):
	try:
		if event == 1:
			NFO = ProcessNFO(Blender.Object.GetSelected())
			SaveNFOFile(NFO)
		elif event == 2:
			SetExportFilename(NFO_FILENAME.val)
			Draw.Redraw()
		elif event == 3:
			global NFO_FILENAME
			Window.FileSelector(SetExportFilename, "Export Directory", NFO_FILENAME.val)
			Draw.Redraw()
		elif event == 4:
			global COPY_IMAGES
			COPY_IMAGES = not COPY_IMAGES
			Draw.Redraw()
		elif event == 5:
			Draw.Exit()
	except StandardError, e:
		print str(e)
		Draw.Exit()
		raise
	
try:
	print inspect.getmembers(Window.FileSelector)
	help(Window.FileSelector)
	Draw.Register(GUIDraw, GUIInputEvent, GUIButtonEvent)	
except StandardError, e:
	print str(e)
	raise

